/*
 * The Spar Library - modular math parser
 * Copyright (C) 2000,2001 Davide Angelocola <davide178@inwind.it>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 *
 */

#ifdef HAVE_EXPM1
#include <math.h>
#endif

#include <spar/sl_math_library.h>
#include <spar/sl_math.h>

SL_MATH_FUNCTION (sl_expm1)
#ifndef HAVE_EXPM1
{
  if (sl_abs (x) < M_LN2)
    {
      register double i = 1.0;
      register double sum = x;
      register double term = x / 1.0;

      do
	{
	  i++;
	  term *= x / i;
	  sum += term;
	}
      while (sl_abs (term) > sl_abs (sum) * SL_DOUBLE_EPSILON);

      return sum;
    }
  else
    {
      return sl_exp (x) - 1;
    }
}
#else
{
  return expm1 (x);
}
#endif /* HAVE_EXPM1 */
