/*
 * The Spar Library - modular math parser
 * Copyright (C) 2000,2001 Davide Angelocola <davide178@inwind.it>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 */

#include <spar/sl_math_library.h>

inline
SL_MATH_FUNCTION (sl_pow_1)
{
  return x;
}

inline
SL_MATH_FUNCTION (sl_pow_2)
{
  return sl_pow_1 (x) * sl_pow_1 (x);
}

inline
SL_MATH_FUNCTION (sl_pow_3)
{
  return sl_pow_2 (x) * sl_pow_1 (x);
}

inline
SL_MATH_FUNCTION (sl_pow_4)
{
  return sl_pow_2 (x) * sl_pow_2 (x);
}

inline
SL_MATH_FUNCTION (sl_pow_5)
{
  return sl_pow_3 (x) * sl_pow_2 (x);
}

inline
SL_MATH_FUNCTION (sl_pow_6)
{
  return sl_pow_3 (x) * sl_pow_3 (x);
}

inline
SL_MATH_FUNCTION (sl_pow_7)
{
  return sl_pow_4 (x) * sl_pow_3 (x);
}

inline
SL_MATH_FUNCTION (sl_pow_8)
{
  return sl_pow_4 (x) * sl_pow_4 (x);
}

inline
SL_MATH_FUNCTION (sl_pow_9)
{
  return sl_pow_5 (x) * sl_pow_4 (x);
}

inline double
sl_ipow (double y, int n)
{
  double s = 1.;
  unsigned m, j;

  if (n)
    {
      if (n < 0)
	{
	  m = -n;
	  y = 1. / y;
	}
      else
	m = n;
      for (j = 1; j <= m; j <<= 1)
	{
	  if (j & m)
	    s *= y;
	  y *= y;
	}
    }
  return s;

}
