/*
 * The Spar Library - modular math parser
 * Copyright (C) 2000,2001 Davide Angelocola <davide178@inwind.it>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 *
 */

#include <spar/sl_vector.h>

int sl_vector_init (sl_vector v, double x);
int sl_vector_init_p (sl_vector * v, double x);
int sl_vector_get_item (sl_vector * v, sl_size i, double *item);
int sl_vector_set_item (sl_vector * v, sl_size i, double data);

int
sl_vector_init (sl_vector v, double x)
{
  sl_size i;

  for (i = 0; i < v.dim; i++)
    v.data[i] = x;

  return SL_SUCCESS;
}

int
sl_vector_init_p (sl_vector * v, double x)
{
  sl_size i;
  sl_assert (v != NULL);

  for (i = 0; i < v->dim; i++)
    v->data[i] = x;

  return SL_SUCCESS;
}

int
sl_vector_get_item (sl_vector * v, sl_size i, double *item)
{
  if (i < v->dim)
    {
      *item = v->data[i];

      return SL_SUCCESS;
    }
  else
    {
      return SL_ERROR;
    }
}

int
sl_vector_set_item (sl_vector * v, sl_size i, double data)
{
  if (i < v->dim)
    {
      v->data[i] = data;

      return SL_SUCCESS;
    }
  else
    {
      return SL_ERROR;
    }
}

int
sl_vector_copy (sl_vector r, sl_vector v)
{
  if (r.dim == v.dim)
    {
      sl_size i;

      for (i = 0; i < r.dim; i++)
	r.data[i] = v.data[i];

      return SL_SUCCESS;
    }
  else
    {
      return SL_ERROR;
    }
}

int
sl_vector_copy_p (sl_vector * r, sl_vector * v)
{
  sl_assert (r != NULL);
  sl_assert (v != NULL);

  if (r->dim == v->dim)
    {
      sl_size i;

      for (i = 0; i < r->dim; i++)
	r->data[i] = v->data[i];

      return SL_SUCCESS;
    }
  else
    {
      return SL_ERROR;
    }
}

sl_vector *
sl_vector_dup (sl_vector * v)
{
  sl_assert (v != NULL);

  {
    sl_size i;
    sl_vector *new = sl_vector_new (v->dim);

    for (i = 0; i < v->dim; i++)
      new->data[i] = v->data[i];
    return new;
  }
}
